package com.aitrainer.api.controller

import com.aitrainer.api.model.CustomerTrainingPlanExercise
import com.aitrainer.api.repository.CustomerTrainingPlanExerciseRepository
import org.slf4j.LoggerFactory
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class CustomerTrainingPlanExerciseController(private val customerTrainingPlanExerciseRepository: CustomerTrainingPlanExerciseRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @PostMapping("/customer_training_plan_exercise")
    fun createNewCustomerTrainingPlanExercise(@Valid @RequestBody customerTrainingPlanExercise: CustomerTrainingPlanExercise): ResponseEntity<CustomerTrainingPlanExercise> {
        logger.info("Create customer property: $customerTrainingPlanExercise")
        return ResponseEntity.ok().body(customerTrainingPlanExerciseRepository.save(customerTrainingPlanExercise))
    }

    @GetMapping("/customer_training_plan_exercise/{id}")
    fun getAllByCustomerId(@PathVariable(value = "id") customerId: Long): ResponseEntity<List<CustomerTrainingPlanExercise>> {

        val exerciseList: List<CustomerTrainingPlanExercise>  = customerTrainingPlanExerciseRepository.findAllByCustomerId(customerId)
        logger.info("Get all training plan exercise list by customerId")

        return if(exerciseList.isNotEmpty())
            ResponseEntity.ok().body(exerciseList) else
            ResponseEntity.notFound().build()
    }
}