package com.aitrainer.api.controller

import com.aitrainer.api.ApiApplication
import com.aitrainer.api.model.Configuration
import com.aitrainer.api.repository.ConfigurationRepository
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.RestController
import java.io.File
import java.sql.Connection
import java.sql.DriverManager
import java.sql.ResultSet
import java.util.Properties

@RestController
object Singleton {

    private lateinit var connection: Connection
    private lateinit var configurationRepository: ConfigurationRepository
    private lateinit var properties: ApplicationProperties

    private var initialized: Boolean = false

    private val logger = LoggerFactory.getLogger(ApiApplication::class.simpleName)

    // to use the singleton functionality avoiding the db access before each request
    private var dbVersion: String = ""
    private var appVersion: String = ""

    fun checkDBUpdate(configurationRepository: ConfigurationRepository, properties: ApplicationProperties) {

        if ( dbVersion.isNotEmpty() && appVersion.isNotEmpty())  {
            // no db access
            //println("DB up-to-date, no DB access")
            return
        }

        this.configurationRepository = configurationRepository
        this.properties = properties

        val dbConfig: Configuration = configurationRepository.findByConfigKey("db_version")
        val applicationVersion: String = properties.getVersion()

        this.dbVersion = dbConfig.configValue
        this.appVersion = applicationVersion

        logger.info("DBVersion: " + this.dbVersion + " App version: " + this.appVersion)

        if ( dbConfig.configValue != applicationVersion ) {

            try {


                val versionNumber: String = applicationVersion.replace(".", "_")
                val fileName = "update_$versionNumber.sql"
                val path = "./data/db/"
                val file2Update = path+fileName

                var sqlRows = ""
                File(file2Update).forEachLine {
                    sqlRows += it
                    if ( sqlRows.contains(";") ) {
                        execSQL(sqlRows)
                        sqlRows = ""
                    }
                }
                this.connection.commit()
                logger.info("Database has been updated to $applicationVersion" )
            } catch (exception: Exception) {
                logger.info("Database exception of $applicationVersion: " + exception.message )
                this.connection.rollback()
            }
        }

    }

    private fun execSQL( sql: String ) {
        if (! this.initialized ) {
            this.getConnection()
        }

        with(this.connection) {
            createStatement().execute(sql)
            //commit()
        }
    }

    fun execQuery( sql: String ): ResultSet {
        if (! this.initialized ) {
            this.getConnection()
        }

        return connection.createStatement().executeQuery(sql)
    }

    private fun  getConnection(): Connection {
        val connectionProps = Properties()
        connectionProps["user"] = this.properties.getDatasourceUsername()
        connectionProps["password"] = this.properties.getDatasourcePassword()
        this.connection = DriverManager.getConnection(this.properties.getDatasourceUrl(), connectionProps)
        this.connection.autoCommit = false
        this.initialized = true
        return this.connection
    }
}