package com.aitrainer.api.controller.diet
import com.aallam.openai.api.BetaOpenAI
import com.aitrainer.api.controller.OpenAIController
import com.aitrainer.api.model.diet.*
import com.aitrainer.api.repository.diet.DietRepository
import com.aitrainer.api.repository.diet.MealRepository
import com.aitrainer.api.service.DietFunctions.getDateFromDayName
import com.aitrainer.api.service.DietFunctions.normalizeMealName
import com.aitrainer.api.service.Meals
import com.google.gson.Gson
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.async
import kotlinx.coroutines.runBlocking
import org.json.JSONObject
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*

@RestController
@RequestMapping("/api")
class DietController(private val dietRepository: DietRepository, private val mealRepository: MealRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)
    private var openAiKey: String = ""

    @PostMapping("/diet", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun insert(@RequestBody diet: Diet): ResponseEntity<*> {
        return ResponseEntity.ok().body(dietRepository.save(diet))
    }

    @PostMapping("/diet/{id}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun update(@PathVariable(value = "id") id: Long, @RequestBody diet: Diet): ResponseEntity<Diet> {
        val existingDiet  = dietRepository.findByDietId(id) ?: return ResponseEntity.notFound().build()

        val updatedDiet: Diet = existingDiet.copy(
            dietId = diet.dietId,
            dietUserId = diet.dietUserId,
            dietText = diet.dietText,
            startDate = diet.startDate
        )
        diet.meals.forEach {
            it.diet = diet
            updatedDiet.meals.add(it)
        }
        return ResponseEntity.ok().body(dietRepository.save(updatedDiet))
    }

    @PostMapping("/diet/no_text/{id}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun updateNoText(@PathVariable(value = "id") id: Long, @RequestBody diet: Diet): ResponseEntity<Diet> {
        val existingDiet  = dietRepository.findByDietId(id) ?: return ResponseEntity.notFound().build()

        val updatedDiet: Diet = existingDiet.copy(
            dietId = diet.dietId,
            dietUserId = diet.dietUserId,
            startDate = diet.startDate
        )
        diet.meals.forEach {
            it.diet = diet
            updatedDiet.meals.add(it)
        }
        return ResponseEntity.ok().body(dietRepository.save(updatedDiet))
    }

    @GetMapping("/diet/{dietUserId}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun getByDietUserId(@PathVariable dietUserId: Long): ResponseEntity<List<Diet>> {
        val list = dietRepository.findByDietUserId(dietUserId)

        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }

    @OptIn(BetaOpenAI::class, DelicateCoroutinesApi::class)
    @PostMapping("/diet/generate_premium/{dietUserId}/{test}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun generatePremiumDiet(@PathVariable dietUserId: Long, @PathVariable test: Boolean = false, @RequestBody input: String, @Value("\${openai.key}") openaiKey: String): ResponseEntity<*> {

        val now = LocalDateTime.now()
        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
        val todayString = now.format(formatter)
        var diet: Diet? = dietRepository.findTheLastEmptyDiet(dietUserId)
        val dietSaved: Diet?
        this.openAiKey = openaiKey
        if (diet == null) {
            diet = Diet(
                dietUserId = dietUserId,
                dietText = "",
                startDate = todayString,
                premium = 0,
                generating = 1
            )
            diet = dietRepository.save(diet)

            var result: String?


            if (!test) {
                val systemInput =
                    "Te egy táplálkozási szakértő vagy. A feladat egy étrend elkészítése ebben a 3 lépésben:\n" +
                            "\n" +
                            "1. Vizsgáld meg az input adatokat, és készítsd el az étrendet\n" +
                            "2. Ellenőrizd az ételeket, hogy tényleg létezik-e olyan, és megfelel az input adatok feltételeinek\n" +
                            "3. Ellenőrizd a kimeneti formátumot. Csak a megadott JSON kimeneti formátumban válaszolj, semmilyen más formátum nem elfogadott.\n"


                val deferred = GlobalScope.async {
                    OpenAIController().chat(
                        modelName = "gpt-4",
                        temperature = 0.9,
                        userInput = input,
                        systemInput = systemInput,
                        openaiKey = openaiKey
                    )
                }
                runBlocking {
                    result = deferred.await().toString()
                    result = extractJson(result!!)
                    println("extracted json: $result")
                }

                if (result == null || result!!.isEmpty()) {
                    return ResponseEntity.badRequest().build<String>()
                }
            } else {
                result = "{\n" +
                        "  \"DIET\": [\n" +
                        "    {\n" +
                        "      \"nameDay\": \"monday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"zabkása\", \"mandulatej\", \"málna\", \"méz\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"monday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"grillezett csirkemell\", \"quinoa\", \"spenót\", \"cherry paradicsom\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"monday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"sárgarépa\", \"hummusz\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"monday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"csicseriborsó-saláta\", \"paprika\", \"paradicsom\", \"uborka\", \"olívaolaj\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"tuesday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"rántotta\", \"sonka\", \"paradicsom\", \"gluténmentes kenyér\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"tuesday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"grillezett lazac\", \"barnarizs\", \"zöldbab\", \"citrom\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"tuesday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"natúr joghurt (laktózmentes)\", \"méz\", \"dió\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"tuesday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"paradicsomos quinoa\", \"spárga\", \"tonhal (konzerv)\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"wednesday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"zabkása\", \"mandulatej\", \"eper\", \"magok (napraforgó, tökmag)\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"wednesday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"töltött paprika\", \"rizs\", \"sovány darált pulykahús\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"wednesday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"cékla\", \"avokádó\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"wednesday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"kókusztejes csirke\", \"spenót\", \"barnarizs\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"thursday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"natúr joghurt (laktózmentes)\", \"mogyoróvaj\", \"banán\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"thursday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"görög saláta\", \"olívaolaj\", \"feta sajt (laktózmentes)\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"thursday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"mandula\", \"szárított füge\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"thursday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"grillezett hal\", \"brokkoli\", \"kukoricalisztből készült polenta\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"friday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"gluténmentes müzli\", \"mandulatej\", \"áfonya\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"friday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"lecsó\", \"kolbász (gluténmentes)\", \"gluténmentes kenyér\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"friday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"yoghurt (lactose free)\", \"honey\", \"walnut\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"friday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"grillezett padlizsán\", \"couscous(gluten free)\", \"csicseriborsó\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"saturday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"omlette\", \"sonka\", \"spenót\", \"gluténmentes kenyér\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"saturday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"húsgombóc\", \"paradicsomos mártás\", \"cukkinispirál\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"saturday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"narancs\", \"keksz (gluténmentes)\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"saturday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"saláta\", \"paradicsom\", \"uborka\", \"pulykamell\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"sunday\",\n" +
                        "      \"mealTime\": \"breakfast\",\n" +
                        "      \"meals\": [\"zabkása\", \"mandulatej\", \"mazsola\", \"mogyoró\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"sunday\",\n" +
                        "      \"mealTime\": \"lunch\",\n" +
                        "      \"meals\": [\"grillezett csirkemell\", \"háromszínű quinoa\", \"répa\", \"zöldborsó\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"sunday\",\n" +
                        "      \"mealTime\": \"snack\",\n" +
                        "      \"meals\": [\"natúr joghurt (laktózmentes)\", \"zabpehely\", \"banán\"]\n" +
                        "    },\n" +
                        "    {\n" +
                        "      \"nameDay\": \"sunday\",\n" +
                        "      \"mealTime\": \"dinner\",\n" +
                        "      \"meals\": [\"töltött gomba\", \"couscous(gluten free)\", \"paradicsomsalsa\"]\n" +
                        "    }\n" +
                        "  ]\n" +
                        "}"
            }


            diet.dietText = result!!
            diet.startDate = todayString
            diet.premium = 1

            dietSaved = dietRepository.save(diet)
        } else {
            dietSaved = diet
        }
        val gson = Gson()


        val dietJson = JSONObject(diet.dietText)
        val mealPlansJson = dietJson.getJSONArray("DIET")

        val dietCustoms = mutableListOf<DietCustom>()

        for (i in 0 until mealPlansJson.length()) {
            val dietCustom = gson.fromJson(mealPlansJson.get(i).toString(), DietCustom::class.java)
            dietCustoms.add(dietCustom)
        }


        dietCustoms.forEach { dietCustom ->
            val dietCustomChanged = addTemporaryQuantity(dietCustom)
            var index = 0
            dietCustomChanged.meals.forEach {meal ->
                val mealId: Long = getMealIdFromCustom(meal, dietCustomChanged, index, false, diet.dietUserId )
                val dietMeal = DietMeal(
                    mealId = mealId,
                    meal = meal,
                    mealName =  dietCustom.nameDay + "|" + dietCustom.mealTime,
                    mealDate = getDateFromDayName(dietCustom.nameDay, dietCustom.mealTime),
                    quantity = dietCustom.quantities[index],
                )

                dietMeal.diet = dietSaved
                dietSaved.meals.add(dietMeal)

            }
            index++
        }

        dietSaved.generating = 0

        val dietSavedWithMeals = dietRepository.save(dietSaved)
        return ResponseEntity.ok().body(dietSavedWithMeals)
    }

    fun extractJson(input: String): String {
        val jsonStart = input.indexOf("{")
        val jsonEnd = input.lastIndexOf("}")

        return if(jsonStart != -1 && jsonEnd != -1) {
            input.substring(jsonStart, jsonEnd + 1)
        } else {
            ""
        }
    }

    fun addTemporaryQuantity(dietCustom: DietCustom): DietCustom {
        for (i in 0 until dietCustom.meals.size) {
            var sum = 400
            when (dietCustom.mealTime) {
                Meals.breakfast.toString() -> sum = 400
                Meals.elevenses.toString() -> sum = 100
                Meals.lunch.toString() -> sum = 500
                Meals.snack.toString() -> sum = 150
                Meals.dinner.toString() -> sum = 350
                Meals.eveningsnack.toString() -> sum = 100
            }

            if (i == 0) {
                dietCustom.quantities.add(sum * 0.7)
            } else {
                dietCustom.quantities.add(sum * 0.3 / (dietCustom.meals.size - 1))
            }
        }
        return dietCustom
    }

    fun getMealIdFromCustom(mealName: String, dietCustom: DietCustom, mealIndex: Int, onlyQuantity: Boolean, dietUserId: Long): Long {
        var mealId: Long = 0
        var foundMeal: Meal? = mealRepository.findByName(mealName)
        var dietCustomVar = dietCustom
        if (foundMeal == null) {
            dietCustomVar = addTemporaryQuantity(dietCustomVar)

            if (!onlyQuantity) {
                foundMeal = getMealCaloriesFromAI(mealName, dietCustomVar.quantities[mealIndex])
            }

            if (foundMeal != null) {
                mealId = foundMeal.id
            }
        } else {
            println("FoundMeal cals: ${foundMeal.calMin}")
            mealId = foundMeal.id
        }
        return mealId
    }

    @OptIn(BetaOpenAI::class, DelicateCoroutinesApi::class)
    fun getMealCaloriesFromAI(mealName: String, quantity: Double): Meal? {
        var meal: Meal? = null

        val systemInput = "Te egy táplálkozási szakértő vagy. Feladat az ételek kalóriaértékének meghatározása"

        val input = """
    Add meg ennek az ételnek a kalóriaértékét és tápanyagfelosztását: $mealName 100 gramm. 
    A választ ebben JSON formátumban írd, ne fűzz semmilyen megjegyzést hozzá: {
      "name":<string>, 
      "quantity":<double>, 
      "quantityUnit":<string>, 
      "calMin":<double>,
      "calMax":<double>, 
      "fatMin":<double>, 
      "fatMax":<double>, 
      "chMin":<double>, 
      "chMax":<double>, 
      "proteinMin":<double>, 
      "proteinMax":<double>, 
      "sugar":<double>, 
      "servingUnit":<enum(db, szelet, adag, ml, liter)> 
      "serving":<double, servingUnit in grams> } 
    """
        val response: String?
        val openaiKey = this.openAiKey
        val deferred = GlobalScope.async {
            OpenAIController().chat(
                modelName = "gpt-3.5-turbo",
                temperature = 0.0,
                userInput = input,
                systemInput = systemInput,
                openaiKey = openaiKey)
        }
        runBlocking {
            response = deferred.await().toString()
        }

        try {
            val newMeal = Gson().fromJson(response, Meal::class.java)
            val newMealName = newMeal.name
            val found: Meal? = mealRepository.findByName(newMealName)
            if (found == null) {
                val baseQuantity = 100.0

                    val rate = quantity / baseQuantity
                    newMeal.quantity = baseQuantity

                    newMeal.calMin /= rate
                    newMeal.calMax /= rate
                    newMeal.chMin /= rate
                    newMeal.chMax /= rate
                    newMeal.fatMin /= rate
                    newMeal.fatMax /= rate
                    newMeal.proteinMin /= rate
                    newMeal.proteinMax /= rate
                    newMeal.sugar /= rate
                    newMeal.normalizedName = normalizeMealName(newMeal.name)
                    newMeal.serving = newMeal.serving
                    newMeal.servingUnit = newMeal.servingUnit
                    val savedMeal = mealRepository.save(newMeal)
                    println("save new meal: ${newMeal.name}, ${newMeal.calMin} kCal")
                    //updateServingForMeal(savedMeal)
                    meal = savedMeal

            } else {
                meal = found
            }
        } catch (e: Exception) {
            println( "Error in newMeal decode JSON: '$e'")
        }
        return meal
    }

    @OptIn(BetaOpenAI::class, DelicateCoroutinesApi::class)
    fun updateServingForMeal(meal: Meal) {


        val input = """
    Add meg, hány gramm egy adag "${meal.name}" átlagosan. 
    A választ pontosan ebben a formátumban kérem: {"meal": <string>, "serving":<double (average in gramms)>, "servingUnit": <enum (adag, szelet, db, ml, liter)>}. 
    The response MUST NOT contain other words just the JSON.
    """

        val response: String?
        val openaiKey = this.openAiKey
        val deferred = GlobalScope.async {
            OpenAIController().chat(modelName = "gpt-3.5-turbo", temperature = 0.0, userInput = input, openaiKey = openaiKey)
        }
        runBlocking {
            response = deferred.await().toString()
        }

        val serving = Gson().fromJson(response, MealServing::class.java)
        meal.serving = serving.serving
        meal.servingUnit = serving.servingUnit
        meal.normalizedName = normalizeMealName(meal.name) // placeholder for your function
        mealRepository.save(meal)
    }

}