package com.aitrainer.api.service

import java.time.LocalDateTime
import java.util.*

enum class Meals {
    breakfast,
    elevenses,
    lunch,
    snack,
    dinner,
    eveningsnack;

    companion object {
        private fun Meals.enumToString(): String = this.toString().lowercase(Locale.getDefault())
        fun Meals.equalsStringTo(mealName: String): Boolean = this.toString().lowercase(Locale.getDefault()) == mealName.lowercase(
            Locale.getDefault()
        )

        fun getMealsByIndex(index: Int): String? {
            for (meal in Meals.values()) {
                if (index == meal.ordinal) {
                    return meal.enumToString()
                }
            }
            return null
        }

        private fun Meals.getMealTimeHour(): Int {
            return when (this) {
                breakfast -> 7
                elevenses -> 9
                lunch -> 11
                snack -> 14
                dinner -> 17
                eveningsnack -> 20
            }
        }

        fun getHour(text: String): Int? {
            for (meal in Meals.values()) {
                if (text.lowercase(Locale.getDefault()) == meal.enumToString() || text.lowercase(Locale.getDefault())
                        .contains(meal.enumToString().lowercase(Locale.getDefault()))) {
                    return meal.getMealTimeHour()
                }
            }
            return null
        }

        fun contains(text: String): Meals? {
            for (meal in Meals.values()) {
                if (text.lowercase(Locale.getDefault()).contains(meal.enumToString().lowercase(Locale.getDefault()))) {
                    return meal
                }
            }
            return null
        }
    }
}


enum class WeekDays {
    monday,
    tuesday,
    wednesday,
    thursday,
    friday,
    saturday,
    sunday;

    companion object WeekDaysExt {
        private fun WeekDays.enumToString(): String {
            return this.toString().lowercase(Locale.getDefault())
        }

        fun WeekDays.equalsStringTo(dayName: String): Boolean {
            return this.toString().equals(dayName, ignoreCase = true)
        }

        private fun WeekDays.getWeekdayNumber(): Int {
            return when (this) {
                monday -> 1
                tuesday -> 2
                wednesday -> 3
                thursday -> 4
                friday -> 5
                saturday -> 6
                sunday -> 7
            }
        }

        fun contains(text: String): WeekDays? {
            var actualDay: WeekDays? = null
            for (day in WeekDays.values()) {
                if (text.contains(day.enumToString())) {
                    actualDay = day
                    break
                }
            }
            return actualDay
        }

        fun weekDay(text: String): Int? {
            var actualDay: Int? = null
            for (day in WeekDays.values()) {
                val weekDayHu = day.enumToString()
                if (day.name.equals(text, true) || weekDayHu.equals(text, true)) {
                    actualDay = day.getWeekdayNumber()
                    break
                }
            }
            return actualDay
        }
    }
}

object DietFunctions {
    @Throws(Exception::class)
    fun getDateFromDayName(dayName: String, mealName: String): String {
        var dayNameVar = dayName
        if (dayNameVar == "evening_snack") {
            dayNameVar = "eveningSnack"
        }
        val today = LocalDateTime.now()
        val weekDay = WeekDays.weekDay(dayNameVar)
        val hour = Meals.getHour(mealName)

        weekDay ?: throw Exception("$dayNameVar is not a week day")
        hour ?: throw Exception("$mealName is not a meal")

        val difference = today.dayOfWeek.value - weekDay
        val day = today.minusDays(difference.toLong())

        return "${day.year}-${day.monthValue}-${day.dayOfMonth}  $hour:00"
    }

    fun normalizeMealName(hungarianWord: String): String {

        val diacriticals: Map<Char, Char> = mapOf(
            'á' to 'a',
            'é' to 'e',
            'í' to 'i',
            'ó' to 'o',
            'ö' to 'o',
            'ő' to 'o',
            'ú' to 'u',
            'ü' to 'u',
            'ű' to 'u',
            'Á' to 'A',
            'É' to 'E',
            'Í' to 'I',
            'Ó' to 'O',
            'Ö' to 'O',
            'Ő' to 'O',
            'Ú' to 'U',
            'Ü' to 'U',
            'Ű' to 'U',
            '-' to ' ',
            ',' to ' ',
        )

        var englishWord = ""
        for (element in hungarianWord) {
            englishWord += diacriticals[element] ?: element
        }

        return englishWord.lowercase(Locale.getDefault())
    }

}