package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerController
import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.User
import com.aitrainer.api.repository.CustomerRepository
import com.aitrainer.api.repository.MembershipRepository
import com.aitrainer.api.service.ServiceBeans
import com.fasterxml.jackson.databind.ObjectMapper
import com.google.gson.Gson
import org.json.JSONObject
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpStatus
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.status
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*
import kotlin.test.assertEquals
import kotlin.test.assertNotNull
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@AutoConfigureMockMvc
class CustomerTests {
    @Autowired
    private lateinit var mockMvc: MockMvc

    @Autowired
    private var serviceBean: ServiceBeans? = null

    private var authToken: String? = ""
    @BeforeAll
    fun init() {
        if ( serviceBean == null ) { serviceBean = ServiceBeans() }
    }

    @Autowired
    private lateinit var customerRepository: CustomerRepository

    @Autowired
    private lateinit var  membershipRepository: MembershipRepository


    private var insertedId: Long? = null

    @Test
    fun testGet() {
        val id: Long = 7
        val customer: Customer = customerRepository.findById( id ).orElse(null)
        assertEquals( customer.name, "Átlag 18 éves fiú")

        val id2: Long = 90
        val controller = CustomerController(customerRepository, membershipRepository)
        val response = controller.getCustomerById(id2)

        val customer2: Customer = response.body as Customer
        assertNotNull (customer2)

        assertEquals(customer2.email, "sw@andio.biz")
        assertEquals(customer2.dataPolicyAllowed, 1)
    }

    @Test
    fun testTrainer() {
        val id: Long = 137
        val customers: List<Customer> = customerRepository.findByTrainerId( id )
        assertEquals(customers.count(), 3)
        assertEquals( customers[0].name, "Átlag 13 éves fiú")
    }

    @Test
    fun testInsert() {
        val newCustomer = Customer("Bossanyi", "Tibor", "", 48, "m")
        val savedCustomer: Customer = customerRepository.save(newCustomer)
        assertEquals(savedCustomer.age, 48)

        this.insertedId = savedCustomer.customerId

        val customer: Customer = customerRepository.findById( savedCustomer.customerId ).orElse(null)
        assertEquals( customer.firstname, "Tibor")
        
        this.testUpdate(savedCustomer.customerId)
    }

    fun testUpdate( customerId: Long ) {
        val id: Long = customerId
        assertNotNull(id)

        val updatedCustomer: Customer = customerRepository.findById( id ).orElse(null)

        assertNotNull(updatedCustomer)
        updatedCustomer.firstname ="Tiborka"
        updatedCustomer.dateChange = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"))

        val customer: Customer = customerRepository.save( updatedCustomer )
        assertEquals( customer.firstname, "Tiborka")
        assertTrue(customer.dateChange != null)

        customerRepository.delete(updatedCustomer)
    }

    @Test
    fun testDeactivateCustomer() {
        val id: Long = 90

        val controller = CustomerController(customerRepository, membershipRepository)
        controller.deactivateCustomer(id)

        val customer: Customer = customerRepository.findById(id).orElse(null)
        assertEquals(customer.active, "D")

        customer.active = "Y"
        customerRepository.save(customer)

        val customer2: Customer = customerRepository.findById(id).orElse(null)
        assertEquals(customer2.active, "Y")


    }

    @Test
    fun testFindByEmail() {
        val controller = CustomerController(customerRepository, membershipRepository)
        var response = controller.getCustomerByEmail("sw@andio.biz")

        val customer = response.body
        assertNotNull(customer)
        assertEquals(customer.firstname, "Tib")

        response = controller.getCustomerByEmail("mr@andio.biz")
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)

    }

    @Test
    fun testUpdateCustomer() {
        val id:Long = 90

        val customer: Customer = customerRepository.findById( id ).orElse(null)

        customer.customerId = id
        customer.firstname = "Tib"
        customer.name = "Bossi"
        customer.admin = 1
        customer.active = "Y"
        customer.sex = "m"

        customer.birthYear = 1972
        customer.bodyType = "endomorph"
        customer.dataPolicyAllowed = 1
        customer.fitnessLevel = "advanced"
        customer.dateChange = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"))
        customer.email = "mr@aitrainer.app"
        customer.emailSubscription = 1
        customer.sportId = 2
        customer.firebaseRegToken = "123456789asdfghjkl"
        customer.phone = "123456"
        customer.lifeLong = 1
        customer.lang = "de"


        var updatedCustomer = customerRepository.save(customer)
        assertEquals(updatedCustomer.customerId, 90)
        assertEquals(updatedCustomer.birthYear, 1972)
        assertEquals(updatedCustomer.fitnessLevel, "advanced")
        assertTrue(customer.dateChange != null)
        assertEquals(updatedCustomer.email, "mr@aitrainer.app")
        assertEquals(updatedCustomer.dataPolicyAllowed, 1)
        assertEquals(updatedCustomer.emailSubscription, 1)
        assertEquals(updatedCustomer.firebaseRegToken, "123456789asdfghjkl")
        assertEquals(updatedCustomer.phone, "123456")
        assertEquals(updatedCustomer.lifeLong, 1)
        assertEquals(updatedCustomer.lang, "de")


        customer.email = "sw@andio.biz"
        updatedCustomer = customerRepository.save(customer)
        assertEquals(updatedCustomer.email, "sw@andio.biz")
    }

    @Test
    fun testUpdateCustomer2() {
        var id: Long = 90

        // update no password
        val customer = Customer()

        customer.customerId = id
        customer.password = null
        customer.firstname = "Tib"
        customer.name = "Bossi"
        customer.admin = 1
        customer.sex = "m"
        customer.fitnessLevel = "intermediate"
        //customer.weight = 79
        customer.birthYear = 1972
        customer.dateChange = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"))

        val customerController = CustomerController(customerRepository, membershipRepository)
        var response: ResponseEntity<*> = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        print ("body " + response.body)
        var newCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        //assertEquals(newCustomer?.weight, 79)
        assertEquals(newCustomer!!.firstname, "Tib")
        assertTrue(newCustomer.dateChange != null)
        assertEquals(newCustomer.email, "sw@andio.biz")

        // test not found
        id = 1000
        response = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)


        // update Password
        id = 103
       // customer.password = "blabal"
        customer.firstname = "Kakadu"
        customer.name = "Bos"
        customer.email = "mr@aitrainer.app"
        customer.sportId = 4
        response = customerController.updateCustomerById(id, customer, HttpHeaders.readOnlyHttpHeaders(HttpHeaders.EMPTY) )
        assertEquals(response.statusCode, HttpStatus.OK)
        newCustomer = response.body as Customer
        //assertEquals(newCustomer.password, "blabal")
        assertEquals(newCustomer.firstname, "Kakadu")
        assertEquals(newCustomer.name, "Bos")
        assertEquals(newCustomer.email, "mr@aitrainer.app")
        assertEquals(newCustomer.dataPolicyAllowed, 1)
        assertEquals(newCustomer.sportId, 4)

        val customer2 = newCustomer.copy()
        customer2.email = "sw2@andio.biz"
        customer2.customerId = 103
        customer2.sportId = 0
        val updatedCustomer = customerRepository.save(customer2)
        assertEquals(updatedCustomer.email, "sw2@andio.biz")

    }

    @Test
    fun testClubRegistration(@Value("\${firebase.key}") apiKey: java.lang.String) {
        val json = "{\"firstname\":\"Tib\", \"email\": \"mr@andio.biz\", \"goal\": \"shape\", \"fitnessLevel\": \"advanced\", \"weight\": 85}"
        val controller = CustomerController(customerRepository, membershipRepository)
        val response: ResponseEntity<*> = controller.clubRegistration(json, apiKey)
        assertEquals(response.statusCode, HttpStatus.BAD_REQUEST)
    }

    @Test
    fun testRegistration() {
        val json = "{\"username\":\"bosi2@example.com\",\"password\":\"94385\",\"firebaseUid\":\"3Firebase8Uid1\"}"
        val user: User = User().fromJson(json)
        assertEquals(user.username, "bosi2@example.com")
        val customer = Customer()
        with(customer) {
            email = user.username
            password = user.password
            firebaseUid = user.firebaseUid
        }
        val customerController = CustomerController(customerRepository, membershipRepository)
        customerController.serviceBeans = serviceBean
        val response: ResponseEntity<*> = customerController.registration(json)
        print("body " + response.body)
        val newCustomer: Customer = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        assertEquals(newCustomer.firebaseUid, "3Firebase8Uid1")
        assertEquals(newCustomer.email, "bosi2@example.com")
        assertTrue(newCustomer.dateAdd != null)

/*        val json2 = "{\"username\":\"bosi2@example.com\",\"password\":\"934345\",\"firebaseUid\":\"3Firebase8Uid3\"}"
        response = customerController.registration(json2)
        assertEquals(response.statusCode, HttpStatus.BAD_REQUEST)*/

       customerRepository.delete(newCustomer)
    }

    @Test
    fun testUpdateFirebaseUid() {
        val newCustomer = Customer("Bossanyi2", "Tibor", "", 48, "m")
        val savedCustomer: Customer = customerRepository.save(newCustomer)
        assertEquals(savedCustomer.age, 48)

        insertedId = savedCustomer.customerId

        val customerController = CustomerController(customerRepository, membershipRepository)
        val response: ResponseEntity<*> = customerController.updateCustomerFirebaseUidById(insertedId!!, "3FirebusaeId4")
        val newCustomer2: Customer = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        assertEquals(newCustomer2.firebaseUid, "3FirebusaeId4")
        assertEquals(newCustomer2.firstname, "Tibor")
        assertEquals(newCustomer2.name, "Bossanyi2")
        assertEquals(newCustomer2.age, 48)

        customerRepository.delete(newCustomer2)
    }

    @Test
    fun testGetCustomerByFirebaseUid() {
        val uid = "3FirebaseU1d"
        val customerController = CustomerController(customerRepository, membershipRepository)
        val response: ResponseEntity<*> = customerController.getCustomerByFirebaseUid(uid)
        assertEquals(response.statusCode, HttpStatus.OK)
        val newCustomer: Customer = response.body as Customer
        assertEquals(newCustomer.name, "Bos")
        assertEquals(newCustomer.email, "sw2@andio.biz")

    }

    @Test
    fun testGetCustomerByEmail() {
        val email = "sw2@andio.biz"
        val customerController = CustomerController(customerRepository, membershipRepository)
        val response: ResponseEntity<*> = customerController.getCustomerByEmail(email)
        assertEquals(response.statusCode, HttpStatus.OK)

        val newCustomer: Customer = response.body as Customer

        assertEquals(newCustomer.name, "Bos")
        assertEquals(newCustomer.email, "sw2@andio.biz")
        assertEquals(newCustomer.firebaseUid, "3FirebaseU1d")

    }

    @Test
    fun `insert customer successfully`() {
        authToken = Tokenizer.getToken()

        val customer = Customer(
            name = "Kadarka",
            firstname = "József",
            age = 32,
            dateAdd = "2023-02-09 08:00",
            birthYear = 1987
        )

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/customers")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(customer))
        )
            .andExpect(status().isOk)
            .andExpect(MockMvcResultMatchers.jsonPath("$.name").value("Kadarka"))
            .andExpect(MockMvcResultMatchers.jsonPath("$.age").value(32))
            .andExpect(MockMvcResultMatchers.jsonPath("$.birthYear").value(1987))
            .andReturn()

        val customerId = JSONObject(mvcResult.response.contentAsString).getInt("customerId")
        println("MockCustomer Id $customerId")
        assertTrue(customerId > 0)

        customerRepository.delete(customer)
    }

    @Test
    fun `login successfully`() {
        authToken = Tokenizer.getToken()

        val user = User(
            username = "sw2@andio.biz",
            password = "andio2009",
            firebaseUid = "",
        )

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/login")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(user))
        )
            .andExpect(status().isOk)
            .andExpect(MockMvcResultMatchers.jsonPath("$.name").value("Bos"))
            .andExpect(MockMvcResultMatchers.jsonPath("$.firstname").value("Kakadu"))
            .andExpect(MockMvcResultMatchers.jsonPath("$.birthYear").value(1972))
            .andReturn()

        val customerId = JSONObject(mvcResult.response.contentAsString).getInt("customerId")
        println("MockCustomer Id $customerId")
        assertEquals(customerId, 103)
    }

    @Test
    fun `update password`() {
        authToken = Tokenizer.getToken()

        val id = 103
        var password = Base64.getEncoder().encodeToString("andio20091".toByteArray())

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/customers/update_password/$id")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(password)
        )
        .andExpect(status().isOk)
        .andExpect(MockMvcResultMatchers.jsonPath("$.name").value("Bos"))
        .andExpect(MockMvcResultMatchers.jsonPath("$.firstname").value("Kakadu"))
        .andExpect(MockMvcResultMatchers.jsonPath("$.birthYear").value(1972))
        .andReturn()

        val customerId = JSONObject(mvcResult.response.contentAsString).getInt("customerId")
        println("MockCustomer Id $customerId")
        assertEquals(customerId, 103)

        password = Base64.getEncoder().encodeToString("andio2009".toByteArray())
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/customers/update_password/$id")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(password)
        )
            .andExpect(status().isOk)
    }

    @Test
    fun `new membership successfully`() {
        authToken = Tokenizer.getToken()
        val gson= Gson()

        val customer = getCustomer(103)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/membership/89")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(customer))
        ).andExpect(status().isNotFound)

        val mvcResult2: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/membership/2")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(customer))
        ).andExpect(status().isOk)
            //.andExpect(MockMvcResultMatchers.jsonPath("$.memberships.length").value(3))
            .andReturn()

        val customer2Json = mvcResult2.response.contentAsString
        println(customer2Json)
        val customer2 = gson.fromJson(customer2Json, Customer::class.java)
        println(customer2)
    }

    fun getCustomer(customerId: Long) : Customer {
        authToken = Tokenizer.getToken()

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.get("/api/customers/$customerId")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
        )
            .andExpect(status().isOk)
            .andReturn()

        val gson= Gson()
        val customerJson = mvcResult.response.contentAsString
        return gson.fromJson(customerJson, Customer::class.java)

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }


}