package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.DietUserPreference
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietUserPreferenceTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private val dietUserPreference = DietUserPreference(
        dietPreferenceId = 10,
        dietUserId = 5,
        preference = ""
    )

    private val dietUserPreference2 = DietUserPreference(
        dietPreferenceId = 0,
        dietUserId = 5,
        preference = "Cukorbetegség",
        preferenceType = "betegség"
    )

    private var authToken: String = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert dietUserPreference successfully`() {

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_preference")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserPreference))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(5))
        .andExpect(jsonPath("$.dietPreferenceId").value(10))
        .andExpect(jsonPath("$.preference").value(""))
        .andExpect(jsonPath("$.preferenceType").value(""))

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_preference")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserPreference2))
        )
            .andExpect(status().isOk)

        mockMvc.perform(get("/api/diet_user_preference/{dietUserId}", 5)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].dietPreferenceId").value(10))
            .andExpect(jsonPath("$.[0].preference").value(""))
            .andExpect(jsonPath("$.[1].preference").value("Cukorbetegség"))

        val dietUserPreference3 = DietUserPreference(
            dietPreferenceId = 0,
            dietUserId = 5,
            preference = "Ebéd-Vacsora egyben",
            preferenceType = "egyéb"
        )

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_preference")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserPreference3))
        )
        .andExpect(status().isOk)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_preference/delete")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserPreference3))
        )
            .andExpect(status().isOk)

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
